"""
File: sleepythreads.py

Illustrates concurrency with multiple threads.
"""

import random, time
from threading import Thread

class SleepyThread(Thread):
    """Represents a sleepy thread."""

    def __init__(self, number, sleepMax):
        """Create a thread with the given name 
        and a random sleep interval less than the maximum. """
        Thread.__init__(self, name = "Thread " + str(number))
        self._sleepInterval = random.randint(1, sleepMax)

    def run(self):
        """Print the thread's name and sleep interval and sleep
        for that interval. Print the name again at wakeup. """
        print "%s, sleep interval: %d seconds" % \
              (self.getName(), self._sleepInterval)
        time.sleep(self._sleepInterval)
        print "%s waking up" % self.getName()

def main():
    """Create the user's number of threads with sleep
    intervals less than the user's maximum. Then start
    the threads"""
    numThreads = input("Enter the number of threads: ")
    sleepMax = input("Enter the maximum sleep time: ")
    threadList = []
    for count in xrange(numThreads):
        threadList.append(SleepyThread(count + 1, sleepMax))
    for thread in threadList: thread.start()

main()    
