"""
Solves the single Producer and multiple Consumer synchronization problem
"""
from threading import Thread, currentThread, Condition
import random, time

class SharedCell(object):
    """Shared data for the producer/consumer problem."""

    def __init__(self):
        self._data = -1
        self._writeable = True
        self._condition = Condition()

    def setData(self, data):
        """Producer's method to write to shared data."""
        self._condition.acquire()
        while not self._writeable:
            self._condition.wait()
        print "%s, setting data %d" % \
              (currentThread().getName(), data)
        self._data = data
        self._writeable = False
        self._condition.notify()
        self._condition.release()
        
    def getData(self):
        """Consumer's method to read from shared data."""
        self._condition.acquire()
        while self._writeable:
            self._condition.wait()
        print "%s, accessing data %d" % \
              (currentThread().getName(), self._data)
        self._writeable = True
        self._condition.notify()
        self._condition.release()
        return self._data
        

class Producer(Thread):
    """Represents a producer."""

    def __init__(self, cell, accessCount, sleepMax):
        """Create a producer with the given shared cell,
        number of accesses, and maximum sleep interval"""
        Thread.__init__(self, name = "Producer")
        self._accessCount = accessCount
        self._cell = cell
        self._sleepMax = sleepMax

    def run(self):
        """Announce start-up, sleep, and write to shared cell
        the given number of times, and  announce completion."""
        print "%s, starting up" % self.getName()
        for count in xrange(self._accessCount):
            time.sleep(random.randint(1, self._sleepMax))
            self._cell.setData(count + 1)
        print "%s is done producing" % self.getName()

class Consumer(Thread):
    """Represents a Consumer."""

    def __init__(self, myName, cell, accessCount, sleepMax):
        """Create a consumer with the given shared cell,
        number of accesses, and maximum sleep interval"""
        Thread.__init__(self, name = myName)
        self._accessCount = accessCount
        self._cell = cell
        self._sleepMax = sleepMax

    def run(self):
        """Announce start-up, sleep, and write to shared cell
        the given number of times, and  announce completion."""
        print "%s, starting up" % self.getName()
        for count in xrange(self._accessCount):
            time.sleep(random.randint(1, self._sleepMax))
            value = self._cell.getData()
        print "%s is done consuming" % self.getName()

def main():
    """Get the number of accesses from the user,
    create a shared cell, and create and start up
    a producer and consumer."""
    accessCount = input("Enter the number of accesses: ")
    consumerCount = input("Enter the number of Consumers: ")
    sleepMax = 4
    cell = SharedCell()
    producer = Producer(cell, accessCount*consumerCount, sleepMax)
    consumerList = []
    for count in xrange(consumerCount):
        consumerList.append(Consumer("Consumer"+str(count), cell,
                                     accessCount, sleepMax))
    print "Starting the threads"
    producer.start()
    for consumer in consumerList:
        consumer.start()

main()
temp = raw_input("Hit Enter to quit")
